; *****************************************************************************
;
;                One armed bandit by Paul Robson (c) 1998
;
;       TERM :  starts the reels
;       GO :    stops the reels
;       1..3 :  operates hold
;
;       At present
;               x x ?   scores 5
;               x x x   scores 16..128 (pattern 0 = 16,pattern 7 = 128)
;
;       This program is in two parts and can fit in a standard MK14
;       with 256 bytes expansion RAM. The first part is a tight fit in
;       1/4k
;
; *****************************************************************************

Part1   = 0B00h                         ; part 1
Part2   = 0F20h                         ; part 2
    
#define High(x) ((x) / 256)             ; useful macros
#define Low(x)  ((x) & 255)

Ram     = 0F00h                         ; where the variables go
Disp    = 0D00h                         ; Display
Crom    = 010Bh                         ; 7 Segment Conversion Table
E       = 80h                           ; E as index

DBuffer = 0                             ; 8 byte display buffer (0 = RHS)
Reel1   = 8                             ; Reels - these have values 0..7
Reel2   = 9                             ; bit 7 is set if it is 'held'
Reel3   = 10
Hold    = 11                            ; this is non-zero if holding
ScoreL  = 12                            ; Score (BCD), 2 low digits
ScoreH  = 13                            ; Score (BCD), 1 high digit
FlashCt = 14                            ; Flash counter
FlashOr = 15                            ; set to 0xFF to 'on flash'
Count   = 16                            ; General Counter
Incr    = 17                            ; Roll Increment
ReelDly = 18                            ; Gives the delay after push effect
Key     = 19                            ; Key pressed in rolling flag
HoldCt  = 20                            ; Used as random for hold

        .org    Part1

; *****************************************************************************
;
;                               Program Start
;
; *****************************************************************************

Reset:  ldi     High(Ram)               ; P1 points to variables always
        xpah    1
        ldi     Low(Ram)
        xpal    1
        ldi     High(Display-1)         ; P3 = Display function
        xpah    3
        ldi     Low(Display-1)
        xpal    3
        ldi     0                       ; reset everything
        st      Reel1(1)                ; clear the reels
        st      Reel2(1)
        st      Reel3(1)
        st      Hold(1)                 ; no hold
        st      ScoreL(1)               ; score is reset to 100
        ldi     1
        st      ScoreH(1)
        jmp     GoEntry                 ; wait for TERM

; *****************************************************************************
;
;                               New Roll
;
; *****************************************************************************

NewRoll:
        ldi     0                       ; blank the hold marker
        st      Hold(1)
        st      Key(1)                  ; reset the 'key pressed' flag

Broke:  xppc    3                       ; endless loop if no money left
        ld      ScoreL(1)               ; if no money sit in this loop
        or      ScoreH(1)
        jz      Broke

        ccl                             ; decrement the score
        ld      ScoreL(1)
        dai     99h
        st      ScoreL(1)
        ld      ScoreH(1)
        dai     99h
        st      ScoreH(1)

        ldi     100                     ; reset the reel delay count
        st      ReelDly(1)

Roll:   xppc    3
        ild     HoldCt(1)               ; increment the hold counter
        ld      2(2)                    ; is go pressed ?
        xri     0FFh
        or      Key(1)
        st      Key(1)                  ; key pressed yet ?
        jz      NoKey
        dld     ReelDly(1)              ; finished the countdown
        jz      RollOver
NoKey:
        ldi     5                       ; Increment value = 5
        st      Incr(1)
        ldi     High(Ram)               ; P2 = Reel 1
        xpah    2
        ldi     Low(Reel1)
        xpal    2
Roll1:  ld      (2)                     ; bump the reel
        jp      Roll2                   ; if not held...
        jmp     Roll3
Roll2:  ccl
        add     Incr(1)
        ani     7
Roll3:  st      @1(2)
        ld      Incr(1)                 ; shift Incr1 right.. gives 5,3,1,0
        xri     5
        jz      Set3
        ld      Incr(1)
        sr
Roll4:  st      Incr(1)
        jnz     Roll1
        jmp     Roll                    ; roll again
Set3:   ldi     3
        jmp     Roll4

GoNRoll:jmp     NewRoll
GoEntry:jmp     Entry

; *****************************************************************************
;
;                                   Score it
;
; *****************************************************************************

RollOver:
        ld      Reel1(1)                ; remove all holds
        ani     7Fh
        st      Reel1(1)
        ld      Reel2(1)
        ani     7Fh
        st      Reel2(1)
        ld      Reel3(1)
        ani     7Fh
        st      Reel3(1)

        ld      Reel2(1)                ; first two reels the same ?
        xor     Reel3(1)
        jnz     WaitNext                ; if not, score nothing at all

        ld      Reel1(1)                ; all three the same ?
        xor     Reel2(1)
        jz      AllThree

TwoSame:
        ldi     5                       ; xx? scores 5
        jmp     Score

AllThree:                               ; xxx scores (x+1)*16
        ld      Reel1(1)                ; get the score
        ccl
        adi     1                       ; this is reel 1..8
        rr
        rr
        rr
        rr                              ; equivalent to shift left x 4

Score:  st      Incr(1)                 ; bump score by A

Bump1:  ld      ScoreL(1)               ; increment score mod 999
        ccl
        dai     1
        st      ScoreL(1)
        ld      ScoreH(1)
        dai     0
        ani     0Fh
        st      ScoreH(1)

        ldi     40                      ; display it going up
        st      ReelDly(1)
Show1:  xppc    3
        dld     ReelDly(1)
        jnz     Show1

        dld     Incr(1)                 ; any more to do ?
        jnz     Bump1

; *****************************************************************************
;
;                               Roll Completed
;
; *****************************************************************************

WaitNext:
        ld      HoldCt(1)               ; check if new hold
        ani     7                       ; 1 in 8 times
        jz      IsHold                  ; (if 0 gives 1 eventually)
        ldi     1                       ; (no,gives 0)
IsHold: xri     1
        st      Hold(1)

Entry:

HoldTest:                               ; this loop waits for TERM 1 2 3
        xppc    3                       ; redisplay
        ld      7(2)                    ; term pressed ?
        xri     0FFh
        jnz     GoNRoll                 ; re-roll
        ld      Hold(1)                 ; if no hold,loop back
        jz      HoldTest
        ldi     3                       ; hold check
Hold1:  xae
        ld      E(2)                    ; check key pressed.
        xri     0FFh
        jnz     Holding
        ccl                             ; dec E loop back
        ldi     0FFh
        ade
        jnz     Hold1
        jmp     HoldTest
Holding:
        ldi     3                       ; E is now 0..2 (0 = right most)
        xre
        ccl
        adi     Reel1                   ; E is now an offset to the reel
        xae
        ld      E(1)                    ; set the hold bit
        ori     80h
        st      E(1)
        jmp     HoldTest

; *****************************************************************************
;
;       Convert the Display Data to displayable format, drop through to
;       the display routine
;
; *****************************************************************************

        .org    Part2

Display:
        ild     FlashCt(1)              ; increment and load the flash count
        ani     030h                    ; do we flash this time ?
        jz      NoFlash
        ldi     07Fh                    ; we flash 'on' e.g. to all lit.
NoFlash:xri     07Fh
        st      FlashOr(1)
        ldi     0
        st      DBuffer+7(1)            ; hold position
        st      DBuffer+3(1)            ; the gap
        ld      Hold(1)                 ; are we holding
        jz      NoHold
        ldi     076h                    ; 'H' for hold pattern
        or      FlashOr(1)
        st      DBuffer+7(1)            ; set the hold display on
NoHold:
        ldi     High(Pattern)           ; P2 = Reel Patterns
        xpah    2
        ldi     Low(Pattern)
        xpal    2
        ldi     2                       ; Count = 2
        st      Count(1)
ReelConv:
        ccl
        adi     Reel1                   ; now 80h(1) points to the reel
        xae
        ld      E(1)                    ; get the reel value
        xae
        lde                             ; E and A are the reel value
        ani     7                       ; E is reel, A is pattern
        xae                             ; A is reel, E is pattern
        ani     80h                     ; if bit 7 is set
        jz      NoFlash2
        ld      FlashOr(1)              ; then or the final pattern....
NoFlash2:
        or      E(2)                    ; or with the reel pattern
        xae
        ld      Count(1)
        ccl                             ; buffer address
        adi     DBuffer+4
        xae
        st      E(1)                    ; save the element
        dld     Count(1)
        jp      ReelConv                ; do for all three reels
        ldi     High(Crom)              ; P2 = CRom
        xpah    2
        ldi     Low(Crom)
        xpal    2
        ld      ScoreH(1)               ; convert high byte
        xae
        ld      E(2)
        st      DBuffer+2(1)
        ld      ScoreL(1)               ; convert low byte
        sr
        sr
        sr
        sr
        xae
        ld      E(2)
        st      DBuffer+1(1)
        ld      ScoreL(1)
        ani     0Fh
        xae
        ld      E(2)
        st      DBuffer+0(1)

; *****************************************************************************
;
;                           Display routine
;
; *****************************************************************************

        ldi     High(Disp)              ; P2 = Disp
        xpah    2
        ldi     Low(Disp)
        xpal    2
        ldi     7
DispLoop:                               ; display 7 segments
        xae
        ld      E(1)
        st      E(2)
        dly     1
        ccl
        ade
        jp      DispLoop
        xppc    3                       ; return to caller
        jmp     Display                 ; go round again

; *****************************************************************************
;
;                     Patterns for the 7 reel sequences
;
; *****************************************************************************

Pattern:
        .db     40h                     ; -
        .db     3Fh                     ; 0
        .db     07h                     ; 7
        .db     7Ch                     ; b
        .db     38h                     ; L
        .db     70h                     ; |-
        .db     36h                     ; ||
        .db     49h                     ; 3 horz lines

        .end
